'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _Element2 = require('./Element');

var _Element3 = _interopRequireDefault(_Element2);

var _lines = require('../utils/lines');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Token = function (_Element) {
    (0, _inherits3.default)(Token, _Element);
    (0, _createClass3.default)(Token, null, [{
        key: 'create',

        /**
         * Generic token constructor.
         *
         * @param {String} type
         * @param {*} value
         */
        value: function create(type, value) {
            return new Token(type, value, valueToSourceCode(type, value));
        }

        /**
         * Creates new token using babel/acorn parser token.
         *
         * @param {{type: String, value: String, sourceCode: String}} token
         */

    }, {
        key: 'createFromToken',
        value: function createFromToken(token) {
            return new Token(token.type, token.value, token.sourceCode);
        }

        /**
         * @param {String} type
         * @param {String} value
         * @param {String} _sourceCode private source code argument
         */

    }]);

    function Token(type, value, _sourceCode) {
        (0, _classCallCheck3.default)(this, Token);

        var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(Token).call(this, type, []));

        if (arguments.length === 2) {
            _sourceCode = valueToSourceCode(type, value);
        }

        var isComment = false;
        var isWhitespace = false;
        var isCode = true;
        switch (type) {
            case 'CommentLine':
                isComment = true;
                isCode = false;
                break;
            case 'CommentBlock':
                isComment = true;
                isCode = false;
                break;
            case 'Whitespace':
                isWhitespace = true;
                isCode = false;
                break;
            case 'AppleInstrumentationDirective':
            case 'GritDirective':
            case 'Hashbang':
                isCode = false;
                break;
        }
        _this._value = value;
        _this._sourceCode = _sourceCode;
        _this._sourceCodeLength = _sourceCode.length;
        _this._sourceCodeLines = (0, _lines.getLines)(_sourceCode);
        _this._isComment = isComment;
        _this._isWhitespace = isWhitespace;
        _this._isCode = isCode;
        return _this;
    }

    (0, _createClass3.default)(Token, [{
        key: '_setChildren',
        value: function _setChildren(newChildren) {
            if (newChildren.length > 0) {
                throw new Error('Token nodes cannot contain child nodes');
            }

            this._childElements = newChildren;
        }

        /**
         * Clones current Element structure.
         *
         * @returns {Element}
         */

    }, {
        key: 'cloneElement',
        value: function cloneElement() {
            return new Token(this._type, this._value, this._sourceCode);
        }
    }, {
        key: 'firstToken',
        get: function get() {
            return this;
        }
    }, {
        key: 'lastToken',
        get: function get() {
            return this;
        }
    }, {
        key: 'isToken',
        get: function get() {
            return true;
        }
    }, {
        key: 'isComment',
        get: function get() {
            return this._isComment;
        }
    }, {
        key: 'isCode',
        get: function get() {
            return this._isCode;
        }
    }, {
        key: 'isWhitespace',
        get: function get() {
            return this._isWhitespace;
        }
    }, {
        key: 'value',
        get: function get() {
            return this._value;
        }
    }, {
        key: 'sourceCode',
        get: function get() {
            return this._sourceCode;
        }
    }, {
        key: 'sourceCodeLength',
        get: function get() {
            return this._sourceCodeLength;
        }
    }, {
        key: 'sourceCodeLines',
        get: function get() {
            return this._sourceCodeLines.concat();
        }
    }, {
        key: 'newlineCount',
        get: function get() {
            return this._sourceCodeLines.length - 1;
        }
    }]);
    return Token;
}(_Element3.default);

exports.default = Token;


function valueToSourceCode(type, value) {
    switch (type) {
        case 'CommentLine':
            return '//' + value;

        case 'CommentBlock':
            return '/*' + value + '*/';

        case 'RegularExpression':
            return String(value);

        case 'Numeric':
        case 'Boolean':
        case 'Null':
            return String(value);

        default:
            return value;
    }
}